using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace TiledLevel
{
    class Player : GameObject
    {
        private const float PLAYER_DECEL = 0.02f;       // The amount the player slows by when not pushing left stick.
        private const float PLAYER_ACCEL = 0.008f;      // The amount the player accelerates by.
        private const float PLAYER_MAX_SPEED = 0.35f;   // The fastest speed the player can move at.
        private const float PLAYER_JUMP = 0.35f;        // The initial velocity of the player when they jump.
        private const float PLAYER_GRAVITY = 0.01f;     // Gravitation pull applied to the player.

        private static TextureContainer image = new TextureContainer();

        public Player()
        {
            base.texture = image;

            // Hard code the initial position of the player.
            pos.X = 5;
            pos.Y = 62;
        }

        /// <summary>
        /// Move the player around the given map based on the state of the game pad.
        /// </summary>
        /// <param name="map">Map to move the player on.</param>
        public override void Update(Map map)
        {
            GamePadState pad = GamePad.GetState(PlayerIndex.One);
            KeyboardState keys = Keyboard.GetState();

            // Move the player using the GameObject method Move() and note in
            // which direction we hit a wall.
            Direction hitdirn = base.Move(map);

            // If we hit a direction horizontally set the vector's x component to 0.
            if ((hitdirn & Direction.Left) != 0 || (hitdirn & Direction.Right) != 0)
                vector.X = 0f;

            // If we hit a direction vertically set the vector's x component to 0.
            if ((hitdirn & Direction.Up) != 0 || (hitdirn & Direction.Down) != 0)
                vector.Y = 0f;
            
            // Check if the player is pushing the left stick left or right.
            if (pad.ThumbSticks.Left.X > 0.5f || keys.IsKeyDown(Keys.D))
            {
                // Player is pushing right, increase the x vector.  We use PLAYER_ACCEL + PLAYER_DECEL
                // to compensate for the fact that we always tend the x vector towards zero by PLAYER_DECEL;
                vector.X += PLAYER_ACCEL + PLAYER_DECEL;
                
                // Prevent the x vector from going above the maximum speed.
                if (vector.X > PLAYER_MAX_SPEED)
                    vector.X = PLAYER_MAX_SPEED;
            }
            else if (pad.ThumbSticks.Left.X < -0.5f || keys.IsKeyDown(Keys.A))
            {
                // Player is pushing left, decrease the x vector.  We use PLAYER_ACCEL + PLAYER_DECEL
                // to compensate for the fact that we always tend the x vector towards zero by PLAYER_DECEL;
                vector.X -= PLAYER_ACCEL + PLAYER_DECEL;

                // Prevent the x vector from going above the maximum speed.
                if (vector.X < -PLAYER_MAX_SPEED)
                    vector.X = -PLAYER_MAX_SPEED;
            }

            // Tend the x vector towards zero so when the player
            // isn't pushing the left stick we slow to a stop.
            vector.X = Global.TendToZero(vector.X, PLAYER_DECEL);

            // Always increase the y vector due to gravity.
            vector.Y += PLAYER_GRAVITY;

            if (pad.Buttons.A == ButtonState.Pressed || keys.IsKeyDown(Keys.Space))
            {
                // Only jump if we're currently on the ground.  Gravity will have made
                // us hit something beneath us so the Direction.Down flag should be set.
                if ((hitdirn & Direction.Down) != 0)
                    vector.Y = -PLAYER_JUMP;
            }

            // Update the position of the map so it centres on the player.
            map.SetViewPos(pos);

            base.Update(map);
        }

        public static void LoadGraphicsContent(ContentManager content)
        {
            // Load the player image.
            image.tex = content.Load<Texture2D>("gfx/player");
        }
    }
}
